'use strict';

var os = require('os');
var child_process = require('child_process');
var fs2 = require('fs');
var pathModule = require('path');
var promises = require('fs/promises');
var micromatch = require('micromatch');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var fs2__namespace = /*#__PURE__*/_interopNamespace(fs2);
var pathModule__namespace = /*#__PURE__*/_interopNamespace(pathModule);
var micromatch__default = /*#__PURE__*/_interopDefault(micromatch);

// ../manifest/dist/generateScriptableBanner.mjs
var SCRIPTABLE_BANNER_STATIC_LINES = [
  "Variables used by Scriptable.",
  "These must be at the very top of the file. Do not edit."
];
var SCRIPTABLE_MANIFEST_KEYS = [
  "name",
  "alwaysRunInApp",
  "shareSheetInputs",
  "iconColor",
  "iconGlyph"
];
function validateManifest(manifest) {
  if (!manifest || typeof manifest !== "object") {
    throw new Error("Manifest must be an object");
  }
  const keys = Object.keys(manifest);
  for (const key of keys) {
    if (!SCRIPTABLE_MANIFEST_KEYS.includes(key)) {
      throw new Error(`Invalid manifest key: ${key}`);
    }
  }
  const typedManifest = manifest;
  if (typedManifest.alwaysRunInApp !== void 0 && typeof typedManifest.alwaysRunInApp !== "boolean") {
    throw new Error("alwaysRunInApp must be a boolean");
  }
  if (typedManifest.name !== void 0 && typeof typedManifest.name !== "string") {
    throw new Error("name must be a string");
  }
  if (typedManifest.iconColor !== void 0 && typeof typedManifest.iconColor !== "string") {
    throw new Error("iconColor must be a string");
  }
  if (typedManifest.iconGlyph !== void 0 && typeof typedManifest.iconGlyph !== "string") {
    throw new Error("iconGlyph must be a string");
  }
  if (typedManifest.shareSheetInputs !== void 0) {
    if (!Array.isArray(typedManifest.shareSheetInputs)) {
      throw new Error("shareSheetInputs must be an array");
    }
    const validInputs = ["file-url", "url", "image", "plain-text"];
    for (const input of typedManifest.shareSheetInputs) {
      if (!validInputs.includes(input)) {
        throw new Error(`Invalid share sheet input: ${input}`);
      }
    }
  }
}
function generateManifestText(manifest = {}, noDefaults = false) {
  manifest && validateManifest(manifest);
  const bannerManifest = {
    "always-run-in-app": manifest.alwaysRunInApp,
    "share-sheet-inputs": manifest.shareSheetInputs?.join(", "),
    "icon-color": manifest.iconColor ?? (noDefaults ? "" : "blue"),
    "icon-glyph": manifest.iconGlyph ?? (noDefaults ? "" : "circle")
  };
  return Object.entries(bannerManifest).filter(([, value]) => value !== void 0 && value !== "").map(([key, value]) => `${key}: ${value};`).join(" ");
}
function generateScriptableBanner(manifest, noDefaults = false) {
  manifest && validateManifest(manifest);
  return [...SCRIPTABLE_BANNER_STATIC_LINES, generateManifestText(manifest, noDefaults)].map((line) => `// ${line}`).join("\n") + "\n";
}
var BasePathFinder = class {
  pathMap = /* @__PURE__ */ new Map();
  _addPath(path, source) {
    const evaluation = this.evaluatePath(path);
    const type = this._classifyPath(path);
    const metadata = this._enrichMetadata(evaluation.metadata, path, source);
    if (!this.pathMap.has(path)) {
      this.pathMap.set(path, {
        ...evaluation,
        metadata,
        type
      });
    } else {
      const existing = this.pathMap.get(path);
      if (evaluation.score > existing.score) {
        existing.score = evaluation.score;
        existing.metadata = {
          ...existing.metadata,
          ...metadata,
          source,
          // Update the source when score is better
          sources: [...existing.metadata.sources || [], source]
        };
      }
    }
  }
  evaluatePath(filePath) {
    let score = 0;
    let exists = false;
    let isAccessible = false;
    let metadata = {};
    const isWindowsPath = filePath.includes(pathModule__namespace.win32.sep);
    const isPosixPath = filePath.includes(pathModule__namespace.posix.sep);
    if (!isWindowsPath && !isPosixPath) {
      return { path: filePath, score: -100, exists, isAccessible, metadata };
    }
    try {
      exists = fs2__namespace.existsSync(filePath);
      if (exists) {
        score += 8;
        const stats = fs2__namespace.statSync(filePath);
        isAccessible = true;
        metadata.stats = stats;
        if (stats.isDirectory()) {
          score += 5;
          try {
            const contents = metadata.contents = fs2__namespace.readdirSync(filePath);
            if (contents.some(
              (item) => item.includes("~com~") || item.includes("desktop.ini") || item.includes("iCloud~") || item.includes(".icloud")
            )) {
              score += 15;
              metadata.hasICloudMarkers = true;
            }
          } catch {
            isAccessible = false;
          }
        }
      }
    } catch {
      score -= 100;
    }
    return { path: filePath, score, exists, isAccessible, metadata };
  }
  _formatAppName(name) {
    return name.replace(/[.-]/g, " ").replace(/\s+/g, " ").split(" ").filter(Boolean).map((word) => word.charAt(0).toUpperCase() + word.slice(1)).join(" ");
  }
  isAppStoragePath(path) {
    const basename2 = path.split(pathModule__namespace.sep).pop() || "";
    return basename2.includes("~");
  }
  parseAppName(path) {
    const basename2 = path.split(pathModule__namespace.sep).pop() || "";
    const appId = basename2.includes("~") ? basename2 : "";
    const [_, ...parts] = basename2.split("~");
    const bundleId = parts.join(".");
    const appName = this._formatAppName(parts.pop() || bundleId);
    const vendor = parts.join(".");
    return { appId, appName, bundleId, vendor };
  }
};
var PathType = /* @__PURE__ */ ((PathType2) => {
  PathType2["ROOT"] = "root";
  PathType2["APP_STORAGE"] = "app_storage";
  PathType2["PHOTOS"] = "photos";
  PathType2["DOCUMENTS"] = "documents";
  PathType2["OTHER"] = "other";
  return PathType2;
})(PathType || {});
var MacPathFinder = class extends BasePathFinder {
  MOBILE_DOCUMENTS_PATH = "Library/Mobile Documents";
  ICLOUD_ROOT_DIR = "com~apple~CloudDocs";
  async findPaths() {
    try {
      await this._findUserPaths();
      await this._checkContainerPaths();
      return Array.from(this.pathMap.values()).sort((a, b) => b.score - a.score).filter((info) => info.score > 0);
    } catch (error) {
      throw new Error(`Failed to find iCloud paths: ${error instanceof Error ? error.message : String(error)}`);
    }
  }
  _classifyPath(path) {
    const basename2 = path.split("/").pop() || "";
    if (basename2 === this.ICLOUD_ROOT_DIR) {
      return "root";
    }
    if (this.isAppStoragePath(path)) {
      return "app_storage";
    }
    if (basename2.toLowerCase().includes("photos")) {
      return "photos";
    }
    if (basename2.toLowerCase().includes("documents")) {
      return "documents";
    }
    return "other";
  }
  _enrichMetadata(metadata, path, source) {
    const enriched = {
      ...metadata,
      source
    };
    const { appId, appName, bundleId, vendor } = this.parseAppName(path);
    Object.assign(enriched, { appId, appName, bundleId, vendor });
    return enriched;
  }
  async _findUserPaths() {
    try {
      const currentHome = os.homedir();
      if (currentHome) {
        await this._checkUserPath(currentHome);
      }
      const users = await this._getSystemUsers();
      for (const user of users) {
        const userHome = `/Users/${user}`;
        if (userHome !== currentHome && fs2__namespace.existsSync(userHome)) {
          const mobileDocsPath = pathModule.join(userHome, this.MOBILE_DOCUMENTS_PATH);
          if (fs2__namespace.existsSync(mobileDocsPath)) {
            await this._checkUserPath(userHome);
          }
        }
      }
      const sharedPath = "/Users/Shared/CloudDocs";
      if (fs2__namespace.existsSync(sharedPath)) {
        this._addPath(sharedPath, {
          source: "shared",
          directoryType: "shared"
        });
      }
    } catch (error) {
      console.debug("Error finding user paths:", error);
    }
  }
  async _getSystemUsers() {
    try {
      const cmd = 'dscl . -list /Users | grep -v "^_"';
      const output = child_process.execSync(cmd, { encoding: "utf8" });
      return (output || "").trim().split("\n").filter(Boolean);
    } catch (error) {
      console.debug("Error getting system users:", error);
      return [];
    }
  }
  async _checkUserPath(userHome) {
    try {
      const mobileDocs = pathModule.join(userHome, this.MOBILE_DOCUMENTS_PATH);
      const entries = await fs2__namespace.promises.readdir(mobileDocs, { withFileTypes: true });
      const username = userHome.split("/").pop() || "";
      for (const entry of entries) {
        if (!entry.isDirectory()) continue;
        const fullPath = pathModule.join(mobileDocs, entry.name);
        if (entry.name === this.ICLOUD_ROOT_DIR) {
          this._addPath(fullPath, {
            source: "userDirectory",
            user: username,
            directoryType: "root"
          });
          const docsPath = pathModule.join(fullPath, "Documents");
          if (fs2__namespace.existsSync(docsPath)) {
            this._addPath(docsPath, {
              source: "userDirectory",
              user: username,
              directoryType: "documents"
            });
          }
        } else if (this.isAppStoragePath(entry.name)) {
          this._addPath(fullPath, {
            source: "userDirectory",
            user: username,
            directoryType: "appStorage"
          });
        }
      }
    } catch (error) {
      console.debug(`Error checking user path ${userHome}:`, error);
    }
  }
  async _checkContainerPaths() {
    const containersPath = pathModule.join(os.homedir(), "Library/Containers");
    try {
      const containers = await fs2__namespace.promises.readdir(containersPath, { withFileTypes: true });
      for (const container of containers) {
        if (container.isDirectory() && (container.name.includes("iCloud") || container.name.includes("CloudDocs"))) {
          const dataPath = pathModule.join(containersPath, container.name, "Data/Library/Mobile Documents");
          this._addPath(dataPath, {
            source: "container",
            container: container.name,
            type: "application"
          });
        }
      }
    } catch {
    }
  }
};
var WindowsPathFinder = class extends BasePathFinder {
  async findPaths() {
    try {
      await this._findInCommonLocations();
      await this._findInRegistry();
      await this._discoverAppStoragePaths();
      return Array.from(this.pathMap.values()).sort((a, b) => b.score - a.score).filter((info) => info.score > 0);
    } catch (error) {
      console.debug("Error finding paths:", error);
      return [];
    }
  }
  async _findInCommonLocations() {
    const userProfile = process.env.USERPROFILE;
    if (!userProfile) {
      console.debug("USERPROFILE environment variable not found");
      return;
    }
    const pathsToCheck = [
      // User profile locations
      pathModule.join(userProfile, "iCloudDrive"),
      pathModule.join(userProfile, "iCloud Drive"),
      // Root locations
      "C:\\iCloudDrive",
      "C:\\iCloud Drive",
      "C:\\iCloud\\iCloudDrive",
      // Other common locations
      ...this._getAvailableDrives().flatMap((drive) => [
        `${drive}\\iCloudDrive`,
        `${drive}\\iCloud Drive`,
        `${drive}\\iCloud\\iCloudDrive`
      ])
    ];
    for (const path of pathsToCheck) {
      if (await this._isValidICloudPath(path)) {
        this._addPath(path, { source: "commonPath" });
      }
    }
  }
  async _isValidICloudPath(path) {
    try {
      if (!fs2__namespace.existsSync(path)) return false;
      const stats = fs2__namespace.statSync(path);
      if (!stats.isDirectory()) return false;
      const contents = fs2__namespace.readdirSync(path);
      return contents.some((item) => item === ".icloud" || item === "desktop.ini" || this.isAppStoragePath(item));
    } catch {
      return false;
    }
  }
  async _findInRegistry() {
    const regPaths = [
      "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\SyncRootManager",
      "HKEY_CURRENT_USER\\Software\\Apple Inc.\\iCloud",
      "HKEY_LOCAL_MACHINE\\SOFTWARE\\Apple Inc.\\iCloud",
      "HKEY_LOCAL_MACHINE\\SOFTWARE\\WOW6432Node\\Apple Inc.\\iCloud"
    ];
    for (const regPath of regPaths) {
      try {
        const cmd = `reg query "${regPath}" /ve 2>nul`;
        child_process.execSync(cmd, { stdio: "ignore" });
        const output = child_process.execSync(`reg query "${regPath}" /s`, {
          encoding: "utf8",
          stdio: ["pipe", "pipe", "ignore"]
        });
        this._parseRegistryOutput(output);
      } catch {
      }
    }
  }
  _parseRegistryOutput(output) {
    const lines = output.split("\r\n");
    for (const line of lines) {
      const match = line.trim().match(/REG_\w+\s+([A-Z]:\\[^"]+)/i);
      if (!match) continue;
      const path = match[1];
      if (path.toLowerCase().includes("icloud") && fs2__namespace.existsSync(path)) {
        this._addPath(path, { source: "registry" });
      }
    }
  }
  _classifyPath(path) {
    const normalizedPath = path.toLowerCase();
    path.split("\\").pop() || "";
    if (this.isAppStoragePath(path)) {
      return "app_storage";
    }
    if (normalizedPath.includes("photos")) {
      return "photos";
    }
    if (normalizedPath.includes("documents")) {
      return "documents";
    }
    if (normalizedPath.includes("iclouddrive") || normalizedPath.includes("icloud drive")) {
      return "root";
    }
    return "other";
  }
  async _discoverAppStoragePaths() {
    const rootPaths = Array.from(this.pathMap.values()).filter(
      (info) => info.type === "root" && info.exists && info.isAccessible
    );
    for (const rootPath of rootPaths) {
      try {
        const entries = await fs2__namespace.promises.readdir(rootPath.path, { withFileTypes: true });
        for (const entry of entries) {
          if (!entry.isDirectory()) continue;
          if (this.isAppStoragePath(entry.name)) {
            const appPath = pathModule.join(rootPath.path, entry.name);
            this._addPath(appPath, {
              source: "appStorage",
              rootPath: rootPath.path
            });
          }
        }
      } catch (error) {
        console.debug(`Error scanning root path ${rootPath.path}:`, error);
      }
    }
  }
  _getAvailableDrives() {
    const drives = [];
    try {
      for (let i = 67; i <= 90; i++) {
        const drive = String.fromCharCode(i) + ":";
        if (fs2__namespace.existsSync(drive)) {
          drives.push(drive);
        }
      }
    } catch (error) {
      console.debug("Error getting available drives:", error);
    }
    return drives;
  }
  _enrichMetadata(metadata, path, source) {
    const enriched = {
      ...metadata,
      source
    };
    const { appId, appName, bundleId, vendor } = this.parseAppName(path);
    Object.assign(enriched, { appId, appName, bundleId, vendor });
    return enriched;
  }
};
var ICloudDriveFinder = class {
  finder;
  currentPlatform;
  constructor() {
    this.currentPlatform = os.platform();
    if (this.currentPlatform === "darwin") {
      this.finder = new MacPathFinder();
    } else if (this.currentPlatform === "win32") {
      this.finder = new WindowsPathFinder();
    } else {
      throw new Error("Unsupported platform: " + this.currentPlatform);
    }
  }
  async findPaths(options) {
    const defaultOptions = {
      types: Object.values(PathType),
      includeInaccessible: false,
      minScore: 0
    };
    const finalOptions = { ...defaultOptions, ...options };
    const paths = await this.finder.findPaths();
    return this._filterPaths(paths, finalOptions);
  }
  _filterPaths(paths, options) {
    let filtered = paths.filter((path) => {
      if (!options.includeInaccessible && !path.isAccessible) {
        return false;
      }
      if (options.minScore && path.score < options.minScore) {
        return false;
      }
      if (options.types && !options.types.includes(path.type)) {
        return false;
      }
      return true;
    });
    if (options.appNamePattern) {
      filtered = this._findMatchingApps(options.appNamePattern, filtered);
    }
    return filtered;
  }
  _findMatchingApps(pattern, paths) {
    const searchTerms = pattern.toLowerCase().split(/\s+/);
    return paths.map((path) => ({
      path,
      score: this._calculateMatchScore(searchTerms, path)
    })).filter((result) => result.score > 0).sort((a, b) => b.score - a.score).map((result) => result.path);
  }
  _calculateMatchScore(searchTerms, path) {
    let score = 0;
    const { appName, appId, bundleId } = path.metadata;
    const nameLower = appName?.toLowerCase() || "";
    const idLower = appId?.toLowerCase() || "";
    const bundleLower = bundleId?.toLowerCase() || "";
    for (const term of searchTerms) {
      if (nameLower === term) score += 100;
      else if (nameLower.includes(term)) score += 50;
      if (bundleLower.includes(term)) score += 30;
      if (idLower.includes(term)) score += 20;
    }
    return score;
  }
};
var iCloudDriveFinder = new ICloudDriveFinder();
async function findICloudDrivePaths(options) {
  if (process.platform !== "darwin" && process.platform !== "win32") {
    throw new Error("Unsupported platform: " + process.platform);
  }
  return await iCloudDriveFinder.findPaths(options);
}

// src/logger.ts
var levels = {
  verbose: 0,
  info: 1,
  warn: 2,
  error: 3
};
var Logger = class {
  namespace;
  level;
  enabled;
  constructor(namespace, options = {}) {
    this.namespace = namespace;
    this.level = options.level || "info";
    this.enabled = options.enabled ?? true;
  }
  shouldLog(msgLevel) {
    return this.enabled && levels[msgLevel] >= levels[this.level];
  }
  verbose(...args) {
    if (this.shouldLog("verbose")) {
      console.log(`[${this.namespace}]`, ...args);
    }
  }
  info(...args) {
    if (this.shouldLog("info")) {
      console.log(`[${this.namespace}]`, ...args);
    }
  }
  warn(...args) {
    if (this.shouldLog("warn")) {
      console.warn(`[${this.namespace}]`, ...args);
    }
  }
  error(...args) {
    if (this.shouldLog("error")) {
      console.error(`[${this.namespace}]`, ...args);
    }
  }
};
function createLogger(namespace, options) {
  return new Logger(namespace, options);
}
function createPatternMatcher(patterns) {
  if (!patterns) return () => true;
  const patternArray = Array.isArray(patterns) ? patterns : [patterns];
  const includePatterns = patternArray.filter((p) => !p.startsWith("!"));
  const excludePatterns = patternArray.filter((p) => p.startsWith("!")).map((p) => p.slice(1));
  return (fileName) => {
    return micromatch__default.default.isMatch(fileName, includePatterns, {
      dot: true,
      ignore: excludePatterns
    });
  };
}
async function waitForFile(filePath, timeout = 5e3) {
  const start = Date.now();
  while (Date.now() - start < timeout)
    if (fs2.existsSync(filePath)) {
      try {
        await promises.readFile(filePath, "utf-8");
        return;
      } catch {
        await new Promise((resolve2) => setTimeout(resolve2, 100));
      }
    }
  throw new Error(`File not available after ${timeout}ms: ${filePath}`);
}
function isPathEqual(path1, path2) {
  return pathModule.resolve(path1) === pathModule.resolve(path2);
}

// src/scriptable-deploy.ts
var findICloudBugsUrl = "https://github.com/hexxspark/findicloud/issues";
async function tryReadManifest(filePath, extensions) {
  for (const ext of extensions) {
    try {
      const manifestPath = filePath.replace(/\.[^.]+$/, "") + ext;
      const content = await promises.readFile(manifestPath, "utf-8");
      return JSON.parse(content);
    } catch {
      continue;
    }
  }
  return null;
}
function scriptableDeploy(options = {}) {
  const {
    verbose = false,
    continueOnError = true,
    scriptableDir,
    patterns = ["**/*.{ts,tsx,js,jsx}"],
    addBanner = true,
    manifestExtensions = [".manifest.json", ".manifest", ".json"]
  } = options;
  const filterFile = createPatternMatcher(patterns);
  const logger = createLogger("scriptable-deploy", {
    level: verbose ? "verbose" : "info"
  });
  let targetDir;
  return {
    name: "scriptable-deploy",
    setup(build) {
      if (!build.initialOptions.metafile) {
        logger.verbose("Automatically enabling metafile option");
        build.initialOptions.metafile = true;
      }
      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            const [found] = await findICloudDrivePaths({ appNamePattern: "Scriptable", types: [PathType.APP_STORAGE] });
            targetDir = found?.path;
          }
          if (!targetDir) {
            throw new Error(
              `Scriptable directory not found in iCloud Drive. Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. If the issue persists, please report it at: ${findICloudBugsUrl}`
            );
          }
          logger.verbose(`Target directory: ${targetDir}`);
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${error.message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          logger.error(message);
        }
      });
      build.onEnd(async (result) => {
        if (!result.metafile || !targetDir) {
          logger.error("Missing required configuration");
          return;
        }
        try {
          const outputs = Object.keys(result.metafile.outputs).filter((file) => {
            const { entryPoint } = result.metafile.outputs[file];
            return entryPoint && filterFile(entryPoint);
          });
          for (const file of outputs) {
            const output = result.metafile.outputs[file];
            let content;
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((f) => isPathEqual(f.path, file));
              if (!outputFile) {
                throw new Error(`Output file not found: ${file}`);
              }
              content = new TextDecoder().decode(outputFile.contents);
            } else {
              await waitForFile(file);
              content = await promises.readFile(file, "utf-8");
            }
            let targetFileName = pathModule.basename(file);
            let manifest = null;
            try {
              const inputs = Object.entries(output.inputs);
              if (!inputs.length) {
                throw new Error(`No input files found for ${file}`);
              }
              const sourceFile = inputs[0][0];
              if (!sourceFile) {
                throw new Error(`Invalid source file for ${file}`);
              }
              manifest = await tryReadManifest(sourceFile, manifestExtensions);
              if (manifest?.name) {
                targetFileName = `${manifest.name}.js`;
              }
            } catch (error) {
              logger.warn(`Failed to read manifest for ${file}: ${error}`);
            }
            const targetPath = pathModule.join(targetDir, targetFileName);
            if (addBanner && manifest) {
              try {
                const banner = generateScriptableBanner(manifest);
                const newContent = banner + "\n" + content;
                await promises.writeFile(targetPath, newContent);
                logger.info(`Deployed with banner: ${file} -> ${targetPath}`);
                continue;
              } catch (error) {
                logger.warn(`Failed to add banner to ${file}: ${error}`);
              }
            }
            await promises.writeFile(targetPath, content);
            logger.info(`Deployed: ${file} -> ${targetPath}`);
          }
          logger.info(`Successfully deployed ${outputs.length} files`);
        } catch (error) {
          logger.error(`Deploy failed: ${error.message}`);
          if (!continueOnError) {
            throw error;
          }
        }
      });
    }
  };
}

exports.scriptableDeploy = scriptableDeploy;
