'use strict';

var fs = require('fs');
var promises = require('fs/promises');
var micromatch = require('micromatch');
var path = require('path');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var micromatch__default = /*#__PURE__*/_interopDefault(micromatch);

// src/utils.ts
function createPatternMatcher(patterns) {
  if (!patterns) return () => true;
  const patternArray = Array.isArray(patterns) ? patterns : [patterns];
  const includePatterns = patternArray.filter((p) => !p.startsWith("!"));
  const excludePatterns = patternArray.filter((p) => p.startsWith("!")).map((p) => p.slice(1));
  return (fileName) => {
    return micromatch__default.default.isMatch(fileName, includePatterns, {
      dot: true,
      ignore: excludePatterns
    });
  };
}
function isMatch(fileName, pattern, options) {
  return micromatch__default.default.isMatch(fileName, pattern, options);
}
async function waitForFile(filePath, timeout = 5e3) {
  const start = Date.now();
  while (Date.now() - start < timeout)
    if (fs.existsSync(filePath)) {
      try {
        await promises.readFile(filePath, "utf-8");
        return;
      } catch {
        await new Promise((resolve2) => setTimeout(resolve2, 100));
      }
    }
  throw new Error(`File not available after ${timeout}ms: ${filePath}`);
}
function normalizePath(path) {
  return path.replace(/\\/g, "/");
}
var np = normalizePath;
function isPathEqual(path1, path2) {
  return path.resolve(path1) === path.resolve(path2);
}

exports.createPatternMatcher = createPatternMatcher;
exports.isMatch = isMatch;
exports.isPathEqual = isPathEqual;
exports.normalizePath = normalizePath;
exports.np = np;
exports.waitForFile = waitForFile;
